package ru.dbotthepony.mc.otm.block

import net.minecraft.core.BlockPos
import net.minecraft.world.level.BlockGetter
import net.minecraft.world.level.block.Block
import net.minecraft.world.level.block.EntityBlock
import net.minecraft.world.level.block.entity.BlockEntity
import net.minecraft.world.level.block.state.BlockState
import net.minecraft.world.level.block.state.StateDefinition
import net.minecraft.world.level.block.state.properties.BooleanProperty
import net.minecraft.world.level.material.Material
import net.minecraft.world.level.material.MaterialColor
import net.minecraft.world.phys.shapes.BooleanOp
import net.minecraft.world.phys.shapes.CollisionContext
import net.minecraft.world.phys.shapes.Shapes
import net.minecraft.world.phys.shapes.VoxelShape
import ru.dbotthepony.mc.otm.block.entity.MatterCableBlockEntity
import ru.dbotthepony.mc.otm.block.entity.StorageCableBlockEntity

abstract class CableBlock(properties: Properties) : Block(properties) {
	private val shapes = getShapeForEachState {
		val shapes = ArrayList<VoxelShape>()

		if (it.getValue(CONNECTION_SOUTH)) {
			shapes.add(
				Shapes.box(
					0.5 - 0.15,
					0.5 - 0.15,
					0.65,
					0.5 + 0.15,
					0.5 + 0.15, 1.0
				)
			)
		}

		if (it.getValue(CONNECTION_NORTH)) {
			shapes.add(
				Shapes.box(
					0.5 - 0.15,
					0.5 - 0.15, 0.0,
					0.5 + 0.15,
					0.5 + 0.15,
					0.35
				)
			)
		}

		if (it.getValue(CONNECTION_DOWN)) {
			shapes.add(
				Shapes.box(
					0.5 - 0.15, 0.0,
					0.5 - 0.15,
					0.5 + 0.15,
					0.5 - 0.15,
					0.5 + 0.15
				)
			)
		}

		if (it.getValue(CONNECTION_UP)) {
			shapes.add(
				Shapes.box(
					0.5 - 0.15,
					0.5 - 0.15,
					0.5 - 0.15,
					0.5 + 0.15, 1.0,
					0.5 + 0.15
				)
			)
		}

		if (it.getValue(CONNECTION_EAST)) {
			shapes.add(
				Shapes.box(
					0.65,
					0.5 - 0.15,
					0.5 - 0.15, 1.0,
					0.5 + 0.15,
					0.5 + 0.15
				)
			)
		}
		if (it.getValue(CONNECTION_WEST)) {
			shapes.add(
				Shapes.box(
					0.0,
					0.5 - 0.15,
					0.5 - 0.15,
					0.35,
					0.5 + 0.15,
					0.5 + 0.15
				)
			)
		}

		var finalShape = CORE_SHAPE

		for (add_shape in shapes) {
			finalShape = Shapes.joinUnoptimized(finalShape, add_shape, BooleanOp.OR)
		}

		return@getShapeForEachState finalShape
	}

	init {
		registerDefaultState(defaultBlockState()
			.setValue(CONNECTION_SOUTH, false)
			.setValue(CONNECTION_WEST, false)
			.setValue(CONNECTION_EAST, false)
			.setValue(CONNECTION_NORTH, false)
			.setValue(CONNECTION_UP, false)
			.setValue(CONNECTION_DOWN, false))
	}

	@Suppress("OVERRIDE_DEPRECATION")
	override fun getShape(
		p_60555_: BlockState,
		p_60556_: BlockGetter,
		p_60557_: BlockPos,
		p_60558_: CollisionContext
	): VoxelShape {
		return shapes[p_60555_] ?: CORE_SHAPE
	}

	override fun hasDynamicShape() = true

	override fun createBlockStateDefinition(builder: StateDefinition.Builder<Block, BlockState>) {
		builder.add(
			CONNECTION_SOUTH,
			CONNECTION_WEST,
			CONNECTION_EAST,
			CONNECTION_NORTH,
			CONNECTION_UP,
			CONNECTION_DOWN
		)
	}

	companion object {
		protected val CORE_SHAPE: VoxelShape = Shapes.box(
			0.5 - 0.15,
			0.5 - 0.15,
			0.5 - 0.15,
			0.5 + 0.15,
			0.5 + 0.15,
			0.5 + 0.15
		)

		val CONNECTION_SOUTH: BooleanProperty = BooleanProperty.create("connect_south")
		val CONNECTION_WEST: BooleanProperty = BooleanProperty.create("connect_west")
		val CONNECTION_EAST: BooleanProperty = BooleanProperty.create("connect_east")
		val CONNECTION_NORTH: BooleanProperty = BooleanProperty.create("connect_north")
		val CONNECTION_UP: BooleanProperty = BooleanProperty.create("connect_up")
		val CONNECTION_DOWN: BooleanProperty = BooleanProperty.create("connect_down")

		val MAPPING_CONNECTION_PROP = listOf(
			CONNECTION_DOWN,
			CONNECTION_UP,
			CONNECTION_NORTH,
			CONNECTION_SOUTH,
			CONNECTION_WEST,
			CONNECTION_EAST
		)
	}
}

class MatterCableBlock : CableBlock(
	Properties.of(Material.STONE, MaterialColor.METAL).requiresCorrectToolForDrops().strength(1.0f, 6.0f)),
	EntityBlock {

	override fun newBlockEntity(blockPos: BlockPos, blockState: BlockState): BlockEntity {
		return MatterCableBlockEntity(blockPos, blockState)
	}
}

class StorageCableBlock : CableBlock(
	Properties.of(Material.STONE, MaterialColor.METAL).requiresCorrectToolForDrops().strength(1.0f, 6.0f)),
	EntityBlock {

	override fun newBlockEntity(blockPos: BlockPos, blockState: BlockState): BlockEntity {
		return StorageCableBlockEntity(blockPos, blockState)
	}
}
